@@ -8,6 +8,7 @@ these are not really related, formatting could be factored out as
 a separate base
 """
 
+import sys
 import copy
 
 from collections import MutableSet, Sized, Set
@@ -176,6 +177,10 @@ class Tag(object):
         # type: () -> None
         self.value = None
 
+    def __repr__(self):
+        # type: () -> Any
+        return '{0.__class__.__name__}({0.value!r})'.format(self)
+
 
 class CommentedBase(object):
     @property
@@ -389,7 +394,7 @@ class CommentedSeq(list, CommentedBase):
         for list_index in sorted(self.ca.items, reverse=True):
             if list_index < idx:
                 break
-            self.ca.items[list_index+1] = self.ca.items.pop(list_index)
+            self.ca.items[list_index + 1] = self.ca.items.pop(list_index)
 
     def pop(self, idx=None):
         # type: (Any) -> Any
@@ -398,14 +403,14 @@ class CommentedSeq(list, CommentedBase):
         for list_index in sorted(self.ca.items):
             if list_index < idx:
                 continue
-            self.ca.items[list_index-1] = self.ca.items.pop(list_index)
+            self.ca.items[list_index - 1] = self.ca.items.pop(list_index)
         return res
 
     def _yaml_get_column(self, key):
         # type: (Any) -> Any
         column = None
         sel_idx = None
-        pre, post = key-1, key+1
+        pre, post = key - 1, key + 1
         if pre in self.ca.items:
             sel_idx = pre
         elif post in self.ca.items:
@@ -472,7 +477,7 @@ class CommentedKeySeq(tuple, CommentedBase):
         # type: (Any) -> Any
         column = None
         sel_idx = None
-        pre, post = key-1, key+1
+        pre, post = key - 1, key + 1
         if pre in self.ca.items:
             sel_idx = pre
         elif post in self.ca.items:
@@ -677,8 +682,8 @@ class CommentedMap(ordereddict, CommentedBase):
             if level >= len(key_list):
                 if level > len(key_list):
                     raise IndexError
-                return d[key_list[level-1]]
-            return get_one_level(key_list, level+1, d[key_list[level-1]])
+                return d[key_list[level - 1]]
+            return get_one_level(key_list, level + 1, d[key_list[level - 1]])
 
         try:
             return get_one_level(key, 1, self)
@@ -934,3 +939,22 @@ class CommentedSet(MutableSet, CommentedMap):
     def __repr__(self):
         # type: () -> str
         return 'set({0!r})'.format(self.odict.keys())
+
+
+def dump_comments(d, name='', sep='.', out=sys.stdout):
+    """
+    recurisively dump domments all but the toplevel preceded by the path
+    in dotted form x.0.a
+    """
+    if isinstance(d, dict):
+        if name:
+            print(name)
+        print(d.ca, file=out)
+        for k in d:
+            dump_comments(d[k], name=name + sep + k if name else k, sep=sep, out=out)
+    elif isinstance(d, list):
+        if name:
+            print(name)
+        print(d.ca, file=out)
+        for idx, k in enumerate(d):
+            dump_comments(k, name=name + sep + str(idx) if name else k, sep=sep, out=out)
