@@ -94,7 +94,8 @@ class YAML(object):
             self.Composer = ruamel.yaml.composer.Composer
             self.Constructor = ruamel.yaml.constructor.BaseConstructor
         elif self.typ == 'unsafe':
-            self.Emitter = ruamel.yaml.emitter.Emitter
+            self.Emitter = ruamel.yaml.emitter.Emitter if pure or CEmitter is None \
+                else CEmitter
             self.Representer = ruamel.yaml.representer.Representer
             self.Parser = ruamel.yaml.parser.Parser if pure or CParser is None else CParser
             self.Composer = ruamel.yaml.composer.Composer
@@ -118,7 +119,7 @@ class YAML(object):
         self.version = None
         self.preserve_quotes = None
         self.allow_duplicate_keys = False  # duplicate keys in map, set
-        self.encoding = None
+        self.encoding = 'utf-8'
         self.explicit_start = None
         self.explicit_end = None
         self.tags = None
@@ -382,7 +383,10 @@ class YAML(object):
             delattr(self, "_serializer")
             delattr(self, "_emitter")
         if transform:
-            fstream.write(transform(stream.getvalue()))   # type: ignore
+            val = stream.getvalue()  # type: ignore
+            if self.encoding:
+                val = val.decode(self.encoding)
+            fstream.write(transform(val))
         return None
 
     def get_serializer_representer_emitter(self, stream, tlca):
@@ -425,7 +429,12 @@ class YAML(object):
                                           default_flow_style=default_flow_style)
                 rslvr.__init__(selfx)
         self._stream = stream
-        dumper = XDumper(stream)
+        dumper = XDumper(stream, default_style=self.default_style,
+                         default_flow_style=self.default_flow_style,
+                         allow_unicode=self.allow_unicode, line_break=self.line_break,
+                         explicit_start=self.explicit_start,
+                         explicit_end=self.explicit_end,
+                         version=self.version, tags=self.tags)
         self._emitter = self._serializer = dumper
         return dumper, dumper, dumper
 
@@ -455,6 +464,7 @@ class YAML(object):
         return res
 
     def register_class(self, cls):
+        # type:(Any) -> None
         """
         register a class for dumping loading
         - if it has attribute yaml_tag use that to register, else use class name
@@ -466,6 +476,7 @@ class YAML(object):
             self.representer.add_representer(cls, cls.to_yaml)
         except AttributeError:
             def t_y(representer, data):
+                # type: (Any, Any) -> Any
                 return representer.represent_yaml_object(
                     tag, data, cls, flow_style=representer.default_flow_style)
 
@@ -474,12 +485,14 @@ class YAML(object):
             self.constructor.add_constructor(tag, cls.from_yaml)
         except AttributeError:
             def f_y(constructor, node):
+                # type: (Any, Any) -> Any
                 return constructor.construct_yaml_object(node, cls)
 
             self.constructor.add_constructor(tag, f_y)
 
 
 def yaml_object(yml):
+    # type: (Any) -> Any
     """ decorator for classes that needs to dump/load objects
     The tag for such objects is taken from the class attribute yaml_tag (or the
     class name in lowercase in case unavailable)
@@ -487,11 +500,13 @@ def yaml_object(yml):
     loading, default routines (dumping a mapping of the attributes) used otherwise.
     """
     def yo_deco(cls):
+        # type: (Any) -> Any
         tag = getattr(cls, 'yaml_tag', '!' + cls.__name__)
         try:
             yml.representer.add_representer(cls, cls.to_yaml)
         except AttributeError:
             def t_y(representer, data):
+                # type: (Any, Any) -> Any
                 return representer.represent_yaml_object(
                     tag, data, cls, flow_style=representer.default_flow_style)
 
@@ -500,6 +515,7 @@ def yaml_object(yml):
             yml.constructor.add_constructor(tag, cls.from_yaml)
         except AttributeError:
             def f_y(constructor, node):
+                # type: (Any, Any) -> Any
                 return constructor.construct_yaml_object(node, cls)
 
             yml.constructor.add_constructor(tag, f_y)
